pragma ComponentBehavior: Bound
import QtQuick
import QtQuick.Controls
import QtQuick.Controls.Fusion

Control {
    id: control

    required property real dayWidth
    property date fromDate: model.fromDate
    required property TimelineListModel model
    property real rowHeight: 32
    property date toDate: model.toDate

    Rectangle {
        id: lodgingList

        anchors.bottom: parent.bottom
        anchors.left: parent.left
        anchors.top: parent.top
        border.color: Fusion.outline(control.palette)
        color: control.palette.base
        width: 100

        ListView {
            ScrollBar.vertical: verticalScroll
            anchors.fill: parent
            boundsBehavior: Flickable.StopAtBounds
            clip: true
            headerPositioning: ListView.OverlayHeader
            model: control.model

            delegate: Column {
                required property string name

                width: ListView.view.width

                Label {
                    anchors.left: parent.left
                    anchors.leftMargin: 6
                    anchors.right: parent.right
                    height: control.rowHeight - 1
                    text: parent.name
                    verticalAlignment: Text.AlignVCenter
                }

                Separator {
                    anchors.left: parent.left
                    anchors.right: parent.right
                    palette: control.palette
                }
            }
            header: Rectangle {
                border.color: Fusion.outline(control.palette)
                color: control.palette.base
                height: timelineList.headerItem.height
                width: parent.width
                z: 2

                Label {
                    anchors.centerIn: parent
                    font.bold: true
                    text: qsTr("Lodging")
                }

                Separator {
                    anchors.bottom: parent.bottom
                    anchors.bottomMargin: 1
                    anchors.left: parent.left
                    anchors.right: parent.right
                    palette: control.palette
                }
            }
        }
    }

    Rectangle {
        anchors.bottom: parent.bottom
        anchors.left: lodgingList.right
        anchors.right: parent.right
        anchors.top: parent.top
        border.color: Fusion.outline(control.palette)
        color: control.palette.base

        ListView {
            id: timelineList

            anchors.fill: parent
            boundsBehavior: Flickable.StopAtBounds
            clip: true
            contentWidth: headerItem.implicitWidth
            flickableDirection: Flickable.AutoFlickDirection
            headerPositioning: ListView.OverlayHeader
            model: control.model

            ScrollBar.horizontal: PermanentScrollBar {
            }
            ScrollBar.vertical: PermanentScrollBar {
                id: verticalScroll

            }
            delegate: TimelineView {
                required property TimelineModel timeline

                dayWidth: control.dayWidth
                fromDate: control.fromDate
                height: control.rowHeight
                model: timeline
                toDate: control.toDate
                viewportWidth: ListView.view.width
                viewportX: ListView.view.contentX

                delegate: Reservation {
                    dragParent: timelineList.contentItem
                }

                Separator {
                    anchors.bottom: parent.bottom
                    anchors.left: parent.left
                    anchors.right: parent.right
                    palette: control.palette
                }
            }
            header: Rectangle {
                border.color: Fusion.outline(control.palette)
                color: control.palette.base
                implicitHeight: headerLayout.implicitHeight + 1
                implicitWidth: headerLayout.implicitWidth
                z: 2

                Column {
                    id: headerLayout

                    TimelineMonthRow {
                        dayWidth: control.dayWidth
                        fromDate: control.fromDate
                        height: 24
                        toDate: control.toDate
                    }

                    Separator {
                        anchors.left: parent.left
                        anchors.right: parent.right
                        palette: control.palette
                    }

                    TimelineDayRow {
                        dayWidth: control.dayWidth
                        fromDate: control.fromDate
                        height: 24
                        toDate: control.toDate
                    }
                }

                Separator {
                    anchors.bottom: parent.bottom
                    anchors.bottomMargin: 1
                    anchors.left: parent.left
                    anchors.right: parent.right
                    palette: control.palette
                }
            }

            Row {
                height: parent.height
                parent: timelineList.contentItem

                Repeater {
                    delegate: Rectangle {
                        required property bool isWeekend

                        color: isWeekend ? palette.alternateBase : "transparent"
                        height: parent.height
                        width: control.dayWidth

                        Separator {
                            anchors.bottom: parent.bottom
                            anchors.right: parent.right
                            anchors.top: parent.top
                            palette: control.palette
                        }
                    }
                    model: TimelineDayModel {
                        fromDate: control.fromDate
                        toDate: control.toDate
                    }
                }
            }
        }

        DropArea {
            property int prevDay
            property int prevLodging

            function addDays(date, days) {
                var newDate = new Date(date);
                newDate.setDate(date.getDate() + days);
                return newDate;
            }

            function findDayAndLodging(drag: DragEvent): list<int> {
                const pos = timelineList.contentItem.mapFromItem(this, drag.x, drag.y);
                const day = Math.floor(pos.x / control.dayWidth);
                const lodging = Math.floor(pos.y / control.rowHeight);
                return [day, lodging];
            }

            function isSpotAvailable(day: int, lodging: int, reservation: Reservation): bool {
                const view = timelineList.itemAtIndex(lodging);
                if (!view) {
                    return false;
                }

                return view.timeline.areDatesAvailable(addDays(control.fromDate, day), addDays(control.fromDate, day + reservation.nights), reservation.reservationId);
            }

            anchors.fill: timelineList
            anchors.leftMargin: -30 * control.dayWidth

            onEntered: function (drag) {
                [prevDay, prevLodging] = findDayAndLodging(drag);
            }
            onPositionChanged: function (drag) {
                let [day, lodging] = findDayAndLodging(drag);
                if ((day != prevDay || lodging != prevLodging) && isSpotAvailable(day, lodging, drag.source)) {
                    prevDay = day;
                    prevLodging = lodging;
                } else {
                    day = prevDay;
                    lodging = prevLodging;
                }
                drag.source.y = lodging * control.rowHeight;
                drag.source.x = (day + .5) * control.dayWidth;
            }
        }
    }
}
